@@ -88,6 +88,7 @@
 #' This approach follows Radosavljevic & Anderson (2014).This setting can be changed by assigning 
 #' other.settings$validation.bg to "partition", which will calculate AUC with respect 
 #' to the validation background only. The default value for other.settings$validation.bg is "full".
+
 #' NOTE: When examining validation AUC and other discrimination metrics, the "full" option will likely 
 #' result in higher performance than for the "partition" option because more and varied background data 
 #' should lead to higher discriminatory power for the model. Users should thus make sure they are correctly
@@ -749,7 +750,7 @@ ENMevaluate <- function(occs, envs = NULL, bg = NULL, tune.args = NULL,
     f <- function(x) enm@predict(x$mod.full, envs, other.settings)
     # necessary to convert levels of envs categoricals to numbers for maxent.jar
     # predictions, else error
-    if(!is.null(categoricals)) {
+    if(!is.null(categoricals) & algorithm == "maxent.jar") {
       for(i in 1:length(categoricals)) {
         lev.df <- terra::levels(envs[[categoricals[i]]])
         lev.df[[1]][,2] <- 1:length(cat.levs[[i]])
