@@ -2,4 +2,4 @@ Sys.setenv("R_TESTS" = "")
 library(testthat)
 library(ENMeval)
 
-test_check("ENMeval")
+testthat::test_check("ENMeval")
